/* Hostloop
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _NESSUSD_HOSTLOOP_H
#define _NESSUSD_HOSTLOOP_H

#define HL_NFS 1
#define HL_DNS 2
#define HL_IP  4
#define HL_PING 8
#define HL_REVLOOKUP 16

#define HL_SUBNET_CLASS_A 1
#define HL_SUBNET_CLASS_B 2
#define HL_SUBNET_CLASS_C 3



void * hls_new_session(char * hostname, int options, int subnet_class);
char * hls_get_next_host(void *);


#endif
