/*
 * dumb
 */

#include <includes.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>


#define NAME "X11-Checker"
#define DESC "Checks, if there is an open X11-Server\n\n\
Risk factor : high"
#define COPYRIGHT "Sebastian Schreiber, GPL"
#define SUMM "Determines if there is an X11-Server with disabled access control of the remote system"


PlugExport int plugin_init(struct arglist *desc);
PlugExport int plugin_init(struct arglist *desc)
{
  plug_set_name(desc, NAME);
  plug_set_description(desc, DESC);
  plug_set_summary(desc, SUMM);
  plug_set_copyright(desc, COPYRIGHT);
  plug_set_category(desc, ACT_GATHER_INFO);
  plug_set_family(desc, "EXTRA PLUGIN");
  return(0);
}

PlugExport int plugin_run(struct arglist * args);	
PlugExport int plugin_run(struct arglist * args)
{
  plug_set_key(args, "TEST", ARG_ARGLIST, args);
  plug_get_key(args, "TEST");
  return(0);
}
