#ifndef HOSTLOOP_IPLOOP_H
#define HOSTLOOP_IPLOOP_H

struct in_addr get_first_ip(struct hls_globals_t *, char * hostname);
struct in_addr get_next_ip(struct hls_globals_t *, struct in_addr ip);
unsigned char get_last_ip_number(struct in_addr ip);
char * get_name_from_ip(struct in_addr ip);
void set_ip_number(struct in_addr *ip, unsigned char value, int pos);
int init_ping();
void send_ping(int soc, struct in_addr addr, int index);
int ping_all_ip(struct in_addr * ips, struct in_addr * ips_ok, int num);
int in_cksum(u_short *, int);


#endif
